/**
 * @jest-environment jsdom
 */
import userEvent from '@testing-library/user-event'

import { render } from './utils'

import QDocument from '@/components/inputs/document/QDocument.vue'
import fakeData from '../cases/Document.mock.js'

describe('QDocument.vue', () => {
	const file = new File(['This is a test file!'], 'Test.txt', { type: 'text/plain' })

	it('Links are disabled and hidden on component load', () => {
		const wrapper = render(QDocument, {
			props: {
				fileProperties: fakeData.simpleUsage().fileProperties,
				versions: fakeData.simpleUsage().versionsObj,
				versionsInfo: fakeData.simpleUsage().versionsInfoArray,
				resourcesPath: fakeData.simpleUsage().resourcesPath
			}
		})

		const deleteLink = wrapper.getAllByTitle('Delete')
		const downloadLink = wrapper.getAllByTitle('Download')
		const attachLink = wrapper.getAllByTitle('Attach')

		expect(deleteLink[0].getAttribute('class')).toBe('dropdown-item disabled')
		expect(downloadLink[0].getAttribute('class')).toBe('dropdown-item disabled')
		expect(wrapper.queryByText('Edit')).toBeNull()
		expect(attachLink[0]).not.toBeNull()
	})

	it('Checks new file is successfully uploaded', async () => {
		const wrapper = render(QDocument, {
			props: {
				fileProperties: fakeData.simpleUsage().fileProperties,
				versions: fakeData.simpleUsage().versionsObj,
				versionsInfo: fakeData.simpleUsage().versionsInfoArray,
				resourcesPath: fakeData.simpleUsage().resourcesPath
			}
		})

		const fileInput = wrapper.getByTestId('file-input')
		await userEvent.upload(fileInput, file)

		const docFile = await wrapper.asyncEmitted('submit-file')
		expect(docFile).toBeTruthy()
	})

	it('Checks file with invalid size doesn\'t get uploaded', async () => {
		const wrapper = render(QDocument, {
			props: {
				maxFileSize: 15,
				fileProperties: fakeData.simpleUsage().fileProperties,
				versions: fakeData.simpleUsage().versionsObj,
				versionsInfo: fakeData.simpleUsage().versionsInfoArray,
				resourcesPath: fakeData.simpleUsage().resourcesPath
			}
		})

		const fileInput = wrapper.getByTestId('file-input')
		await userEvent.upload(fileInput, file)

		const docFile = await wrapper.asyncEmitted('submit-file')
		expect(docFile).not.toBeTruthy()
	})
})
