using System.Linq;

namespace quidgest.uitests.pages;

public class AppPage: PageObject {

    private IWebElement Container => driver.FindElement(By.ClassName("layout-container"));

	public IMenuControl Menu => new VerticalMenuControl(driver, _menuTree);

	private By loginBtnLocator => By.Id("logon-menu-btn");
    private IWebElement loginBtn => Container.FindElement(loginBtnLocator);
    private By avatarLocator => By.Name("user-avatar");
    private IWebElement avatar => Container.FindElement(avatarLocator);

    public AppPage(IWebDriver driver) : base(driver) {
		string url = Configuration.Instance.BaseUrl;
		driver.Navigate().GoToUrl(url);

		wait.Until(c => Container);
	}

	private void WaitForLoading()
	{
		wait.Until(c => Container.GetAttribute("data-loading") != "true");
	}

	public void ClickLogin()
    {
		WaitForLoading();
        loginBtn.Click();
	}

	public bool IsAuthenticated()
	{
        WaitForLoading();
		//Until vue has a global way to determine layout loading state we have to hack it like this
		//making this method only usable for IsAuthenticated == true.
		//Don't use this for asserting IsAuthentication == false, it will throw exception.
		//To test when its ok to remove this add a Thread.Sleep(1000) to LogOn in AccountController
		// to simulate slow database access.
        wait.Until(c => avatar);
        if (Container.FindElements(avatarLocator).Any())
            return true;

		return false;
	}

	//Header
		//logo
		//avatar
	//Menu
	//MainContent
		//breadcrumbs
		//sidebar
	//Footer
		//version

	private readonly static MenuTree _menuTree = DeclareMenuTree();

    private static MenuTree DeclareMenuTree()
    {
        MenuTree res = new MenuTree();
		string module;

		module = "SF3";
		res.AddModule(module);
		res.AddMenu(module, "1", null);
		res.AddMenu(module, "2", null);
        return res;
    }
}