﻿using CSGenio.business;
using CSGenio.framework;
using CSGenio.persistence;
using GenioMVC.Models;
using GenioMVC.Models.Navigation;
using GenioMVC.ViewModels;
using GenioServer.security;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http.Extensions;
using Microsoft.AspNetCore.Mvc;
using Quidgest.Persistence.GenericQuery;
using System.Collections.Specialized;

namespace GenioMVC.Controllers
{
    [Authorize]
	public class HomeController : ControllerBase
	{
        public HomeController(UserContextService userContext): base(userContext)
        {
        }


		private static readonly NavigationLocation ACTION_LSTUSR_EDIT = new NavigationLocation("LISTA_DE_UTILIZADORE37232", "ChangeListProperties", "Home");

		public readonly string EPH_Action_Available_Key = "EPH_Action_Available";
		public readonly string EPH_Action_Form_Key = "EPH_Action_Form";

// USE /[MANUAL SF3 HOME_CONTROLLER_INDEX]/
		[HttpGet]
		[AllowAnonymous]
		public ActionResult IndexAuthenticated()
		{
			var model = new Home_ViewModel(UserContext.Current);
			var isGuestUser = UserContext.Current.User.IsGuest();

			// Load the ViewModel of the Home Page
			model.HomePage_model = new ViewModels.Home.HomePage_ViewModel(UserContext.Current, isGuestUser);
			model.HomePage_model.Load();

			if (!isGuestUser)
			{
				if (UserContext.Current.User.NeedsToChangePassword())
					return RedirectToAction("Profile", "Home");
				if (UserContext.Current.User.NeedsToSetup2FA())
					return RedirectToAction("Change2FA", "Home");
			}

			return JsonOK(model);
		}


		[HttpGet]
		public JsonResult GetHomePages()
		{
			var isGuestUser = UserContext.Current.User.IsGuest();

			var model = new ViewModels.Home.HomePage_ViewModel(UserContext.Current, isGuestUser);
			model.Load();

			return JsonOK(model);
		}

		[HttpPost]
		public ActionResult Bookmarks()
		{
			if (UserContext.Current.User.IsGuest() || UserContext.Current.User.Public)
				return new EmptyResult();

			var cacheKey = string.Format("bookmarks.{0}.{1}", UserContext.Current.User.Name, UserContext.Current.User.Codpsw);
			var model = QCache.Instance.User.Get(cacheKey) as ViewModels.Bookmarks.Bookmarks_ViewModel;
			if (model == null)
			{
				model = new ViewModels.Bookmarks.Bookmarks_ViewModel();
				model.LoadMenus(UserContext.Current);
				QCache.Instance.User.Put(cacheKey, model, TimeSpan.FromMinutes(15));
			}

			return JsonOK(model);
		}

        public class RequestAddBookmarkModel
		{
            public string Module { get; set; }
			public string MenuId { get; set; }
        }

        [HttpPost]
		public JsonResult AddBookmark([FromBody]RequestAddBookmarkModel requestModel)
		{
			string module = requestModel.Module;
			string menuId = requestModel.MenuId;

			var sp = UserContext.Current.PersistentSupport;
			var user = UserContext.Current.User;
			try
			{
				var sqlCheck = new SelectQuery() { noLock = true }
					.Select(SqlFunctions.Count("1"), "count")
					.From(CSGenio.business.Area.AreaUSRCFG)
					.Where(CriteriaSet.And()
						.Equal(CSGenioAusrcfg.FldTipo, "FV")
						.Equal(CSGenioAusrcfg.FldCodpsw, user.Codpsw)
						.Equal(CSGenioAusrcfg.FldModulo, module)
						.Equal(CSGenioAusrcfg.FldId, menuId));

				var values = sp.executeReaderOneColumn(sqlCheck);
				int count = (int)values[0];

				if (count != 0)
					return Json(new { Success = true });

				sp.openTransaction();

				var fav = new CSGenio.business.CSGenioAusrcfg(user, module)
				{
					ValTipo = "FV",
					ValModulo = module,
					ValId = menuId,
					ValCodpsw = user.Codpsw
				};

				fav.insert(sp);

				var cacheKey = string.Format("bookmarks.{0}.{1}", UserContext.Current.User.Name, UserContext.Current.User.Codpsw);
				QCache.Instance.User.Invalidate(cacheKey);

				sp.closeTransaction();
			}
			catch (Exception e)
			{
				sp.rollbackTransaction();
				sp.closeConnection();
				Log.Error("Error on AddBookmark. Message: " + e.Message ?? string.Empty);
				return Json(new { Success = false, Message = Resources.Resources.PEDIMOS_DESCULPA__OC63848 });
			}

			try
			{
				var model = new ViewModels.Bookmarks.Bookmarks_ViewModel();
				model.LoadMenus(UserContext.Current);

				var cacheKey = string.Format("bookmarks.{0}.{1}", UserContext.Current.User.Name, UserContext.Current.User.Codpsw);
				QCache.Instance.User.Put(cacheKey, model, TimeSpan.FromMinutes(15));

				return JsonOK(model);
			}
			catch
			{
				return Json(new { Success = false, Message = Resources.Resources.PEDIMOS_DESCULPA__OC63848 });
			}
		}

		public class RequestRemoveBookmarkModel
		{
            public string BookmarkId { get; set; }
        }

		[HttpPost]
		public JsonResult RemoveBookmark([FromBody]RequestRemoveBookmarkModel requestModel)
		{
			string bookmarkId = requestModel.BookmarkId;
			var sp = UserContext.Current.PersistentSupport;
			var user = UserContext.Current.User;

			try
			{
				sp.openTransaction();

				var fav = CSGenio.business.CSGenioAusrcfg.search(sp, bookmarkId, user);

				fav.delete(sp);

				var cacheKey = string.Format("bookmarks.{0}.{1}", UserContext.Current.User.Name, UserContext.Current.User.Codpsw);
				QCache.Instance.User.Invalidate(cacheKey);

				sp.closeTransaction();
				return Json(new { Success = true, fav_id = fav.ValCodusrcfg });
			}
			catch (Exception e)
			{
				sp.rollbackTransaction();
				sp.closeConnection();
				Log.Error("Error on RemoveBookmark. Message: " + e.Message ?? string.Empty);
				return Json(new { Success = false, Message = Resources.Resources.PEDIMOS_DESCULPA__OC63848 });
			}
		}


		public JsonResult GetAvailableMenus()
		{
			var user = UserContext.Current.User;
			var model = new Menu_ViewModel(UserContext.Current);
			return JsonOK(model);
		}

		private void recreateUser()
		{
			QCache.Instance.User.Invalidate("user." + UserContext.Current.User.Name);
			UserContext.Current.User = null;
		}

		// GET: /Home/ProfileRedirect for Vue
		public ActionResult ProfileRedirect()
		{
			return RedirectToVueRoute("profile");
		}

		// GET: /Home/Profile
		[HttpGet]
		public ActionResult Profile()
		{
			var sp = UserContext.Current.PersistentSupport;
			var user = UserContext.Current.User;

			var profile = new ProfileModel();
			profile.Enable2FAOptions = Configuration.Security.Activate2FA != Auth2FAModes.None;

			try
			{
				sp.openConnection();
				var userValues = CSGenioApsw.search(sp, user.Codpsw, user, new string[] { CSGenioApsw.FldCodpsw.Field, CSGenioApsw.FldNome.Field });

				profile.ValCodpsw = userValues.ValCodpsw;
				profile.ValNome = userValues.ValNome;
			}
			catch
			{
				ModelState.AddModelError("Erro", Resources.Resources.PEDIMOS_DESCULPA__OC63848);
			}
			finally
			{
				sp.closeConnection();
			}

			var status = user.Status;
			if (status == 1)
				ModelState.AddModelError(Resources.Resources.PALAVRA_CHAVE_EXPIRA05120, Resources.Resources.PALAVRA_CHAVE_EXPIRA05120);

			// check if configuracoes.xml have OpenID Connect configured
			OpenIdConnectIdentityProvider oIdIP = new OpenIdConnectIdentityProvider();
			if (oIdIP.Options != null)
				profile.OpenIdConnAuthMethods.Add(oIdIP.Options.Description);

			return JsonOK(profile);
		}

		//
		// POST: /Home/Profile/5
		[HttpPost]
		public ActionResult Profile([FromBody]ProfileModel model)
		{
			bool success = false;
			string error = string.Empty;

			var user = UserContext.Current.User;
			var sp = UserContext.Current.PersistentSupport;

			if (user.Codpsw != model.ValCodpsw)
				error = Resources.Resources.NAO_PODE_ALTERAR_A_P42871;
			else
			{
				try
				{
					sp.openConnection();
					var userOldValues = CSGenioApsw.search(sp, user.Codpsw, user, new string[] { CSGenioApsw.FldPassword.Field, CSGenioApsw.FldSalt.Field, CSGenioApsw.FldPswtype.Field });
					error = PasswordFactory.CheckValidPassToChange(user.Name, model.OldPassword, model.NewPassword, model.ConfirmPassword, userOldValues.ValPassword, userOldValues.ValSalt, userOldValues.ValPswtype, user.Language);
				}
				catch
				{
					error = Resources.Resources.PEDIMOS_DESCULPA__OC63848;
				}
				finally
				{
					sp.closeConnection();
				}
			}

			if (error != string.Empty)
				ModelState.AddModelError(error, error);
			else
			{
				try
				{
					sp.openTransaction();
					model.Save(UserContext.Current);
					sp.closeTransaction();

					SuccessMessage(Resources.Resources.A_SUA_PASSWORD_FOI_A50177);
					success = true;

					//recriar user logado, caso contrário
					if (GlobalFunctions.emptyN(UserContext.Current.User.Status) == 0 && UserContext.Current.User.Status == 1)
						recreateUser();
				}
				catch (Exception e)
				{
					sp.rollbackTransaction();

					if (e is GenioException && (e as GenioException).UserMessage != null)
						ModelState.AddModelError("Erro", (e as GenioException).UserMessage);
					else
						ModelState.AddModelError("Erro", Resources.Resources.PEDIMOS_DESCULPA__OC63848);
					return JsonERROR(null, model);
				}
			}

			model.OldPassword = "";
			model.NewPassword = "";
			model.ConfirmPassword = "";

			if (success)
				return JsonOK(new { Message = Resources.Resources.A_SUA_PASSWORD_FOI_A50177 });
			return JsonERROR(null, model);
		}


		public ActionResult Change2FA()
		{
			var model = new TwoFAViewModel();

			var userPsw = Models.Psw.Find(UserContext.Current.User.Codpsw, UserContext.Current);
			model.HasTotp = userPsw.ValPsw2fatp == Auth2FAModes.TOTP.ToString() ? 1 : 0;
			model.HasWebAuthN = userPsw.ValPsw2fatp == Auth2FAModes.WebAuth.ToString() ? 1 : 0;
			model.ShowTotp = false;

			//give to user a message if is mandatory to create 2FA
			if (Configuration.Security.Mandatory2FA && !UserContext.Current.User.Auth2FA)
				ModelState.AddModelError("Erro", Resources.Resources.A_2ND_AUTHENTICATION36972);

			return JsonOK(model);
		}

		[HttpPost]
		public ActionResult Change2FA([FromBody]TwoFAViewModel model)
		{
			if (model.HasTotp == 1)
			{
				var secret = model.TotpDisplayCode;
				//Only save if the user has correctly inserted the 6 code, otherwise they may be locked out of the system
				if (new TOTPIdentityProvider().IsOk(secret, model.Totp6Code))
				{
					var sp = UserContext.Current.PersistentSupport;
					try
					{
						sp.openConnection();
						var userPsw = Models.Psw.Find(UserContext.Current.User.Codpsw, UserContext.Current);
						userPsw.ValPsw2fatp = Auth2FAModes.TOTP.ToString();
						userPsw.ValPsw2favl = secret;
						userPsw.Save(sp);
						sp.closeConnection();
					}
					catch (Exception ex)
					{
						sp.closeConnection();
						Log.Error(ex.Message);

						ModelState.AddModelError("user", Resources.Resources.PEDIMOS_DESCULPA__OC63848);
						CreateTOTPModel(ref model, secret);
						return JsonERROR(Resources.Resources.PEDIMOS_DESCULPA__OC63848, model);
					}

					recreateUser();
				}
				else
				{
					ModelState.AddModelError(Resources.Resources.THE_CODE_YOU_ENTERED21835, Resources.Resources.THE_CODE_YOU_ENTERED21835);
					CreateTOTPModel(ref model, secret);
					return JsonERROR(Resources.Resources.THE_CODE_YOU_ENTERED21835, model);
				}
			}

			return JsonOK();
		}

		private string getUrlQrCodeTOTP (string secret)
		{
			return TOTPIdentityProvider.GetUrlQrCode(UserContext.Current.User.Name, secret);
		}

		private void CreateTOTPModel(ref TwoFAViewModel model, string secret)
		{
			model.HasTotp = 1;
			model.HasWebAuthN = 0;
			model.ShowTotp = true;

			var qrUrl = getUrlQrCodeTOTP(secret);
			model.TotpUrl = qrUrl;
			model.TotpDisplayCode = secret;
		}

		[HttpGet]
		public ActionResult CreateTOTP ()
		{
			var model = new TwoFAViewModel();

			//Creation 2FA based on TOTP
			string secret = PasswordFactory.StringRandom(20, true);

			//save the 2FA secret
			UserContext.Current.User.Code = secret;

			CreateTOTPModel(ref model, secret);

			return JsonOK(model);
		}

		[HttpGet]
		public ActionResult CreateWebAuthN()
		{
			var model = new TwoFAViewModel();

			model.HasTotp = 0;
			model.HasWebAuthN = 1;
			model.ShowWebAuthN = true;

			return JsonOK(model);
		}

		public ActionResult WebAuthn2FAMakeCredentialOptions()
		{
			WebAuthIdentityProvider credWebAuth = new WebAuthIdentityProvider(new WebAuthValues()
			{
				MDSAccessKey = ModelState.GetValueOrDefault("fido2:MDSAccessKey")?.AttemptedValue,
				MDSCacheDirPath = ModelState.GetValueOrDefault("fido2:MDSCacheDirPath")?.AttemptedValue,
				TimestampDriftTolerance = ModelState.GetValueOrDefault("fido2:TimestampDriftTolerance")?.AttemptedValue,

                //Fido2Options = new WebAuthFido2Options() { Origin = Request.Url.GetLeftPart(UriPartial.Authority) }
                Fido2Options = new WebAuthFido2Options() { Origin = $"{Request.Scheme}://{Request.Host}{Request.PathBase}" }
            });

			var returnWebAuth = credWebAuth.MakeCredentialOptions(UserContext.Current.User.Name);

			if (returnWebAuth.Success)
			{
				//Temporarily store options, session/in-memory cache/redis/db
				HttpContext.Session.SetString("fido2.attestationOptions", returnWebAuth.Options);
				return Json(new { Success = true, options = returnWebAuth.Options });
			}
			else
				return Json(new { Success = false, ErrorMessage = returnWebAuth.ErrorMessage });
		}

		public async Task<ActionResult> WebAuthn2FAMakeCredentialOptions2(string data)
		{
			WebAuthIdentityProvider credWebAuth = new WebAuthIdentityProvider(new WebAuthValues()
			{
				MDSAccessKey = ModelState.GetValueOrDefault("fido2:MDSAccessKey")?.AttemptedValue,
				MDSCacheDirPath = ModelState.GetValueOrDefault("fido2:MDSCacheDirPath")?.AttemptedValue,
				TimestampDriftTolerance = ModelState.GetValueOrDefault("fido2:TimestampDriftTolerance")?.AttemptedValue,

                //Fido2Options = new WebAuthFido2Options() { Origin = Request.Url.GetLeftPart(UriPartial.Authority) }
                Fido2Options = new WebAuthFido2Options() { Origin = $"{Request.Scheme}://{Request.Host}{Request.PathBase}" }
			});

			User u = UserContext.Current.User;
			PersistentSupport sp = PersistentSupport.getPersistentSupport(u.Year, u.Name);
			var returnWebAuth = await credWebAuth.MakeCredential(data, HttpContext.Session.GetString("fido2.attestationOptions"), UserContext.Current.User.Codpsw, sp);

			if (returnWebAuth.Success)
				return Json(new { Success = returnWebAuth.Success, options = returnWebAuth.Options });
			return Json(new { Success = returnWebAuth.Success, ErrorMessage = returnWebAuth.ErrorMessage });
		}

		[HttpPost]
		public ActionResult CreateOpenIdLoginRedirect()
		{
			string urlRedirectAuth = (new OpenIdConnectIdentityProvider()).GetUrlToAuthenticate(
				Url.RouteUrl("OIdRegist", null, Request.Scheme) //Get absolute path with scheme + domain + "/OpenIdRegister" to provider known were to send the callback
			);

			return Json(new { url = urlRedirectAuth });
		}

		[HttpPost]
		public ActionResult OpenIdRegister([FromForm]string id_token, [FromForm] string code)
		{
			var sp = UserContext.Current.PersistentSupport;
			try
			{
				//decode JWT received, more information at https://openid.net/specs/openid-connect-core-1_0.html#IDToken
				var token = new System.IdentityModel.Tokens.Jwt.JwtSecurityToken(id_token);

				OpenIdConnectIdentityProvider ip = new OpenIdConnectIdentityProvider();
				ip.Options.CallbackPath = Url.RouteUrl("OIdRegist", null, Request.Scheme); //Get absolute path with scheme + domain + "/OIdRegist" to provider known were to send the callback
				TokenCredential qToken = new TokenCredential();
				qToken.Token = token.ToString();

				bool validToken = ip.ValidateToken(qToken, code);

				if (validToken) //When user authenticated successfull we will save user info
				{
					// the token it's composed by two JSON. The first one are header and the second one are payload. Here we will use the payload
					dynamic jsonPayload = Newtonsoft.Json.Linq.JObject.Parse(qToken.Token.Substring(qToken.Token.IndexOf("}.{") + 2));

					string username = jsonPayload.sub.Value + //Subject
									"@" + jsonPayload.iss.Value; //Issuer

					//save data to PSW
					sp.openConnection();
					var userPsw = Models.Psw.Find(UserContext.Current.User.Codpsw, UserContext.Current);
					userPsw.ValUserid = username;
					userPsw.Save(sp);
					sp.closeConnection();

					SuccessMessage(Resources.Resources.CONTA_FOI_CRIADA_COM31537);
				}
			}
			catch (Exception ex)
			{
				sp.closeConnection();
				ErrorMessage(ex.Message);
			}

			return RedirectToVuePage("Profile");
		}

		[AllowAnonymous]
		public ActionResult About()
		{
			return JsonOK(/* TODO: data ?? */);
		}

		[AllowAnonymous]
		public ActionResult NavigationalBar()
		{
			var availableMenus = Helpers.Menus.Menus.GetModuleMenus(UserContext.Current, UserContext.Current.User.CurrentModule, true);
			return JsonOK(new
			{
				Module = UserContext.Current.User.CurrentModule,
				MenuList = availableMenus
			});
		}

		[AllowAnonymous]
		public ActionResult NavigationalBarTop()
		{
			var availableMenus = Helpers.Menus.Menus.GetModuleMenus(UserContext.Current, UserContext.Current.User.CurrentModule, true);
			return JsonOK(new
			{
				Module = UserContext.Current.User.CurrentModule,
				MenuList = availableMenus
			});
		}

		[AllowAnonymous]
		public ActionResult QDebug()
		{
			QDebug_ViewModel model = new QDebug_ViewModel(UserContext.Current);
			return JsonOK(model);
		}

		/// <summary>
		/// Created by [SF] at [2017.03.23]
		/// Fazer refresh à pagina
		/// </summary>
		/// <returns></returns>
		public ActionResult RefreshDbPDF()
		{
			string sessionId = HttpContext.Session.Id;
			if (System.IO.File.Exists(AppDomain.CurrentDomain.BaseDirectory + "temp\\loading" + sessionId + ".txt"))
			{
				System.IO.File.Delete(AppDomain.CurrentDomain.BaseDirectory + "temp\\loading" + sessionId + ".txt");
				return JsonOK(new { success = true, loading = true });
			}
			if (System.IO.File.Exists(AppDomain.CurrentDomain.BaseDirectory + "temp\\" + sessionId + ".txt"))
			{
				System.IO.File.Delete(AppDomain.CurrentDomain.BaseDirectory + "temp\\" + sessionId + ".txt");
				return JsonOK(new { success = true, loading = false });
			}

			return JsonERROR();
		}

		/// <summary>
		/// Created by [SF] at [2022.08.04]
		/// Remove files creates in method PrepareFileLink
		/// </summary>
		/// <returns></returns>
		public ActionResult RemoveFileTemp()
		{
			if (!string.IsNullOrEmpty(Navigation.GetStrValue("filename")))
			{
				if (System.IO.File.Exists(AppDomain.CurrentDomain.BaseDirectory + "temp\\" + Navigation.GetStrValue("filename")))
				{
					System.IO.File.Delete(AppDomain.CurrentDomain.BaseDirectory + "temp\\" + Navigation.GetStrValue("filename"));
					Navigation.ClearValue("filename");
				}
			}

			return JsonOK(new { success = true, loading = true });
		}

		/// <summary>
		/// Created by [HTA] at [2019.10.01]
		/// Devolve um link para ser usado com a aplicação da consola do Office Add-In. Usa o stream do pedido (request)
		/// </summary>
		/// <returns>O redirecionamneto para o link a ser usado na aplicação ou para a página de origem em caso de erro</returns>
		public ActionResult PrepareFileLink()
		{
			PersistentSupport userSP = UserContext.Current.PersistentSupport;
			string url = ""; // TODO: Update to use the exclusive addin portal
			string area = Request.Query["area"].First().ToLower();
			string areaPrimarykey = Request.Query["areakey"].First();
			string userAgent = Request.Headers.UserAgent;
			bool isWindows = false;
			if (userAgent.Contains("Windows"))
				isWindows = true;

			try
			{
				CSGenio.business.Area info = CSGenio.business.Area.createArea(area, UserContext.Current.User, UserContext.Current.User.CurrentModule);
				string tablename = info.TableName;
				string field = "";

				foreach (KeyValuePair<string,Field> fields in info.DBFields)
				{
					if (fields.Key.EndsWith("fk"))
					{
						field = fields.Key;
						break;
					}
				}

				SelectQuery query = new SelectQuery()
					.Select("docums", "document")
					.Select("docums", "docpath")
					.Select("docums", "nome")
					.Select("docums", "coddocums")
					.Select("docums", "datamuda")
					.From(tablename).Join("docums", "docums", TableJoinType.Inner).On(CriteriaSet.And().Equal(tablename, field, "docums", "documid"))
					.Where(CriteriaSet.And()
						.Equal(tablename, info.PrimaryKeyName, areaPrimarykey)
						.Equal(tablename, "zzstate", 0)
						.NotEqual("docums", "versao", "CHECKOUT"))
					.OrderBy("docums", "datacria", SortOrder.Descending)
					.OrderBy("docums", "chave", SortOrder.Ascending).Page(1);
				DataMatrix values = userSP.Execute(query);

				if (values.NumRows > 0)
				{
					Byte[] bytes = new byte[0];
					if (Configuration.Files2Disk)
					{
						System.IO.FileInfo fileinfo = new System.IO.FileInfo(Configuration.PathDocuments + "\\" + values.GetString(0, 1));
						int size = (int)fileinfo.Length;
						bytes = new Byte[size];
						System.IO.FileStream fs = new System.IO.FileStream(Configuration.PathDocuments + "\\" + values.GetString(0, 1), System.IO.FileMode.Open);
						fs.Read(bytes, 0, size);
						fs.Flush();
						fs.Close();
					}
					else
						bytes = values.GetBinary(0, 0);

					string fileName = values.GetString(0, 2);
					string documsPrimaryKey = values.GetString(0, 3);
					string timestamp = values.GetDate(0, 4).ToUniversalTime().ToString("s", System.Globalization.CultureInfo.InvariantCulture);

					string tempFile = ".\\temp\\" + documsPrimaryKey + "-" + fileName;
					using (System.IO.FileStream tempFileStream = System.IO.File.OpenWrite(tempFile))
					{
						tempFileStream.Write(bytes, 0, bytes.Length);
					}
					string baseUrl = $"{Request.Scheme}://{Request.Host}{Request.PathBase}";
					tempFile = baseUrl + "\\temp\\" + documsPrimaryKey + "-" + fileName;

					string protocol = "addin:";
					Navigation.SetValue("filename", documsPrimaryKey + "-" + fileName);
					bool openTaskPane = false;
					if (!string.IsNullOrEmpty(Request.Query["openPane"].First()))
						openTaskPane = bool.Parse(Request.Query["openPane"].First());

					// information format: url | File download url | File name | area name | area primary key | docums primary key | timestamp | open task pane (bool) | platform is Windows (bool)
					string link = protocol + url + "?linkfile=" + tempFile + "&filename=" + fileName + "&area=" + area + "&areakey=" + areaPrimarykey + "&documskey=" + documsPrimaryKey + "&timestamp=" + timestamp + "&taskpane=" + openTaskPane + "&win=" + isWindows;

					return Redirect(link);
				}

				return Redirect(Request.GetDisplayUrl());
			}
			catch (Exception)
			{
				if (userSP != null)
					userSP.closeConnection();
				return Redirect(Request.GetDisplayUrl());
			}
		}

		//Create by [TMV] (2020.09.23)
		/// <summary>
		/// Redirects to the eph menu
		/// </summary>
		/// <returns></returns>
		[HttpGet]
		public ActionResult GetEphFormAction(string module = "")
		{
			User user = UserContext.Current.User;
			string actionName = "";

			if (user.EphTofill != null)
			{
				// Get the action for the form id
				string id = user.EphTofill.GetForm(module);

				GenioMVC.Helpers.Menus.MenuEntry menu;

				// Search all branches possible to navigate
				while (!string.IsNullOrEmpty(id))
				{
					menu = GenioMVC.Helpers.Menus.Menus.FindMenu(module, id);

					if (string.IsNullOrEmpty(menu.Controller))
						break;

					actionName = menu.Route_VUE;
					id = menu.ParentId;
				}
			}

			return JsonOK(new { routeName = actionName });
		}

		#region Programmers area...




// USE /[MANUAL SF3 HOME_CONTROLLER_MANUAL]/

		#endregion
	}
}
