﻿using JsonIgnoreAttribute = System.Text.Json.Serialization.JsonIgnoreAttribute;
using Microsoft.AspNetCore.Mvc.Rendering;
using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Specialized;
using System.ComponentModel.DataAnnotations;
using System.Data;
using System.Globalization;

using CSGenio.business;
using CSGenio.framework;
using CSGenio.persistence;
using GenioMVC.Helpers;
using GenioMVC.Models.Exception;
using GenioMVC.Models.Navigation;
using Quidgest.Persistence;
using Quidgest.Persistence.GenericQuery;

namespace GenioMVC.ViewModels.Agent
{
	public class Agent_ViewModel : FormViewModel<Models.Agent>
	{
		[JsonIgnore]
		public override bool HasWriteConditions { get => false; }

		/// <summary>
		/// Reference for the Models MsqActive property
		/// </summary>
		[JsonIgnore]
		public bool MsqActive { get; set; } = false;

		#region Foreign keys

		#endregion

		/// <summary>
		/// Title: "Photo" | Type: "IJ"
		/// </summary>
		[ImageThumbnailJsonConverter(30, 50)]
		public GenioMVC.ViewModels.ImageModel ValPhoto { get; set; }

		/// <summary>
		/// Title: "Name" | Type: "C"
		/// </summary>
		public string ValName { get; set; }

		/// <summary>
		/// Title: "Birthdate" | Type: "D"
		/// </summary>
		public DateTime? ValBirthdat { get; set; }

		/// <summary>
		/// Title: "Email" | Type: "C"
		/// </summary>
		public string ValEmail { get; set; }

		/// <summary>
		/// Title: "Phone number" | Type: "C"
		/// </summary>
		public string ValTelephon { get; set; }

		#region Navigations
		#endregion

		#region Auxiliar Keys for Image controls



		#endregion

		#region Extra database fields



		#endregion

		#region Fields for formulas


		#endregion


		public string ValCodagent { get; set; }

		/// <summary>
		/// FOR DESERIALIZATION ONLY
		/// A call to Init() needs to be made manually invoked after this constructor
		/// </summary>
		[Obsolete("For deserialization only")]
		public Agent_ViewModel() : base(null!) { }

		public Agent_ViewModel(UserContext userContext, bool nestedForm = false) : base(userContext, "FAGENT", nestedForm) { }

		public Agent_ViewModel(UserContext userContext, Models.Agent row, bool nestedForm = false) : base(userContext, "FAGENT", row, nestedForm) { }

		public Agent_ViewModel(UserContext userContext, string id, bool nestedForm = false, string[]? fieldsToLoad = null) : this(userContext, nestedForm)
		{
			this.Navigation.SetValue("agent", id);
			Model = Models.Agent.Find(id, userContext, "FAGENT", fieldsToQuery: fieldsToLoad);
			if (Model == null)
				throw new ModelNotFoundException("Model not found");
			InitModel();
		}

		protected override void InitLevels()
		{
			this.RoleToShow = CSGenio.framework.Role.ROLE_1;
			this.RoleToEdit = CSGenio.framework.Role.ROLE_1;
		}

		#region Form conditions

		public override StatusMessage InsertConditions()
		{
			return InsertConditions(m_userContext);
		}

		public static StatusMessage InsertConditions(UserContext userContext)
		{
			var m_userContext = userContext;
			StatusMessage result = new StatusMessage(Status.OK, "");
			Models.Agent model = new Models.Agent(userContext) { Identifier = "FAGENT" };

			var navigation = m_userContext.CurrentNavigation;
			// The "LoadKeysFormHistory" must be after the "LoadEPH" because the PHE's in the tree mark Foreign Keys to null
			// (since they cannot assign multiple values to a single field) and thus the value that comes from Navigation is lost.
			// And this makes it more like the order of loading the model when opening the form.
			model.LoadEPH("FAGENT");
			if (navigation != null)
				model.LoadKeysFormHistory(navigation, navigation.CurrentLevel.Level);

			var tableResult = model.EvaluateTableConditions(ConditionType.INSERT);
			result.MergeStatusMessage(tableResult);
			return result;
		}

		public override StatusMessage UpdateConditions()
		{
			StatusMessage result = new StatusMessage(Status.OK, "");
			var model = Model;

			var tableResult = model.EvaluateTableConditions(ConditionType.UPDATE);
			result.MergeStatusMessage(tableResult);
			return result;
		}

		public override StatusMessage DeleteConditions()
		{
			StatusMessage result = new StatusMessage(Status.OK, "");
			var model = Model;

			var tableResult = model.EvaluateTableConditions(ConditionType.DELETE);
			result.MergeStatusMessage(tableResult);
			return result;
		}

		public override StatusMessage ViewConditions()
		{
			var model = Model;
			StatusMessage result = model.EvaluateTableConditions(ConditionType.VIEW);
			var tableResult = model.EvaluateTableConditions(ConditionType.VIEW);
			result.MergeStatusMessage(tableResult);
			return result;
		}

		protected override StatusMessage EvaluateWriteConditions(bool isApply)
		{
			Models.Agent model = Model;
			StatusMessage result = new StatusMessage(Status.OK, "");
			return result;
		}

		public StatusMessage EvaluateTableConditions(ConditionType type)
		{
			return Model.EvaluateTableConditions(type);
		}

		#endregion

		#region Mapper

		public override void MapFromModel(Models.Agent m)
		{
			if (m == null)
			{
				CSGenio.framework.Log.Error("Map Model (Agent) to ViewModel (Agent) - Model is a null reference");
				throw new ModelNotFoundException("Model not found");
			}

			try
			{
				ValPhoto = ViewModelConversion.ToImage(m.ValPhoto);
				ValName = ViewModelConversion.ToString(m.ValName);
				ValBirthdat = ViewModelConversion.ToDateTime(m.ValBirthdat);
				ValEmail = ViewModelConversion.ToString(m.ValEmail);
				ValTelephon = ViewModelConversion.ToString(m.ValTelephon);
				ValCodagent = ViewModelConversion.ToString(m.ValCodagent);
			}
			catch (Exception)
			{
				CSGenio.framework.Log.Error("Map Model (Agent) to ViewModel (Agent) - Error during mapping");
				throw;
			}
		}

		/// <summary>
		/// Performs the mapping of field values from the ViewModel to the Model.
		/// </summary>
		/// <exception cref="ModelNotFoundException">Thrown if <paramref name="m"/> is null.</exception>
		public override void MapToModel()
		{
			MapToModel(this.Model);
		}

		/// <summary>
		/// Performs the mapping of field values from the ViewModel to the Model.
		/// </summary>
		/// <param name="m">The Model to be filled.</param>
		/// <exception cref="ModelNotFoundException">Thrown if <paramref name="m"/> is null.</exception>
		public override void MapToModel(Models.Agent m)
		{
			if (m == null)
			{
				CSGenio.framework.Log.Error("Map ViewModel (Agent) to Model (Agent) - Model is a null reference");
				throw new ModelNotFoundException("Model not found");
			}

			try
			{
				m.ValPhoto = ViewModelConversion.ToImage(ValPhoto);
				m.ValName = ViewModelConversion.ToString(ValName);
				m.ValBirthdat = ViewModelConversion.ToDateTime(ValBirthdat);
				m.ValEmail = ViewModelConversion.ToString(ValEmail);
				m.ValTelephon = ViewModelConversion.ToString(ValTelephon);
				m.ValCodagent = ViewModelConversion.ToString(ValCodagent);
			}
			catch (Exception)
			{
				CSGenio.framework.Log.Error($"Map ViewModel (Agent) to Model (Agent) - Error during mapping. All user values: {HasDisabledUserValuesSecurity}");
				throw;
			}
		}

		/// <summary>
		/// Sets the value of a single property of the view model based on the provided table and field names.
		/// </summary>
		/// <param name="fullFieldName">The full field name in the format "table.field".</param>
		/// <param name="value">The field value.</param>
		/// <exception cref="ArgumentNullException">Thrown if <paramref name="fullFieldName"/> is null.</exception>
		public override void SetViewModelValue(string fullFieldName, object value)
		{
			try
			{
				ArgumentNullException.ThrowIfNull(fullFieldName);
				// Obtain a valid value from JsonValueKind that can come from "prefillValues" during the pre-filling of fields during insertion
				var _value = ViewModelConversion.ToRawValue(value);

				switch(fullFieldName)
				{
					case "agent.name":
						this.ValName = ViewModelConversion.ToString(_value);
						break;
					case "agent.birthdat":
						this.ValBirthdat = ViewModelConversion.ToDateTime(_value);
						break;
					case "agent.email":
						this.ValEmail = ViewModelConversion.ToString(_value);
						break;
					case "agent.telephon":
						this.ValTelephon = ViewModelConversion.ToString(_value);
						break;
					case "agent.codagent":
						this.ValCodagent = ViewModelConversion.ToString(_value);
						break;
					default:
						Log.Error($"SetViewModelValue (Agent) - Unexpected field identifier {fullFieldName}");
						break;
				}
			}
			catch (Exception ex)
			{
				throw new FrameworkException(Resources.Resources.PEDIMOS_DESCULPA__OC63848, "SetViewModelValue (Agent)", "Unexpected error", ex);
			}
		}

		#endregion

		/// <summary>
		/// Reads the Model from the database based on the key that is in the history or that was passed through the parameter
		/// </summary>
		/// <param name="id">The primary key of the record that needs to be read from the database. Leave NULL to use the value from the History.</param>
		public override void LoadModel(string id = null)
		{
			try { Model = Models.Agent.Find(id ?? Navigation.GetStrValue("agent"), m_userContext, "FAGENT"); }
			finally { Model ??= new Models.Agent(m_userContext) { Identifier = "FAGENT" }; }

			base.LoadModel();
		}


		public override void Load(NameValueCollection qs, bool editable, bool ajaxRequest = false, bool lazyLoad = false)
		{
			this.editable = editable;
			CSGenio.business.Area oldvalues = null;

			// TODO: Deve ser substituido por search do CSGenioA
			try
			{
				Model = Models.Agent.Find(Navigation.GetStrValue("agent"), m_userContext, "FAGENT");
			}
			finally
			{
				if (Navigation.CurrentLevel.FormMode == FormMode.New || Navigation.CurrentLevel.FormMode == FormMode.Duplicate)
				{
					if (Model == null)
						throw new ModelNotFoundException("Model not found");

					LoadDefaultValues();
				}
				else
				{
					if (Model == null)
						throw new ModelNotFoundException("Model not found");

					oldvalues = Model.klass;
				}
			}

			Model.Identifier = "FAGENT";
			InitModel(qs, lazyLoad);

			if (Navigation.CurrentLevel.FormMode == FormMode.New || Navigation.CurrentLevel.FormMode == FormMode.Edit || Navigation.CurrentLevel.FormMode == FormMode.Duplicate)
			{
				// MH - Voltar calcular as formulas to "atualizar" os Qvalues dos fields fixos
				// Conexão deve estar aberta de fora. Podem haver formulas que utilizam funções "manuais".
				MapToModel(Model);
				// Preencher operações internas
				Model.klass.fillInternalOperations(m_userContext.PersistentSupport, oldvalues);
				MapFromModel(Model);
			}

			// Load just the selected row primary keys for checklists.
			// Needed for submitting forms incase checklists are in collapsible zones that have not been expanded to load the checklist data.
			LoadChecklistsSelectedIDs();
		}

		protected override void FillExtraProperties()
		{
		}

		protected override void LoadDocumentsProperties(Models.Agent row)
		{
		}

		/// <summary>
		/// Load Partial
		/// </summary>
		/// <param name="lazyLoad">Lazy loading of dropdown items</param>
		public override void LoadPartial(NameValueCollection qs, bool lazyLoad = false)
		{
			// MH [bugfix] - Quando o POST da ficha falha, ao recaregar a view os documentos na BD perdem alguma informação (ex: name do file)
			if (Model == null)
			{
				// Precisamos fazer o Find to obter as chaves dos documentos que já foram anexados
				// TODO: Conseguir passar estas chaves no POST to poder retirar o Find.
				Model = Models.Agent.Find(Navigation.GetStrValue("agent"), m_userContext, "FAGENT");
				if (Model == null)
				{
					Model = new Models.Agent(m_userContext) { Identifier = "FAGENT" };
					Model.klass.QPrimaryKey = Navigation.GetStrValue("agent");
				}
				MapToModel(Model);
				LoadDocumentsProperties(Model);
			}
			// Add characteristics
			Characs = new List<string>();

// USE /[MANUAL SF3 VIEWMODEL_LOADPARTIAL AGENT]/
		}

// USE /[MANUAL SF3 VIEWMODEL_NEW AGENT]/

		// Preencher Qvalues default dos fields do form
		protected override void LoadDefaultValues()
		{
		}

		public override CrudViewModelValidationResult Validate()
		{
			CrudViewModelFieldValidator validator = new(m_userContext.User.Language);


			validator.StringLength("ValName", Resources.Resources.NAME31974, ValName, 50);
			validator.Required("ValName", Resources.Resources.NAME31974, ValName);
			validator.StringLength("ValEmail", Resources.Resources.EMAIL25170, ValEmail, 50);
			validator.Required("ValEmail", Resources.Resources.EMAIL25170, ValEmail);
			validator.StringLength("ValTelephon", Resources.Resources.PHONE_NUMBER20774, ValTelephon, 50);

			return validator.GetResult();
		}

// USE /[MANUAL SF3 VIEWMODEL_SAVE AGENT]/
		public override void Save()
		{


			base.Save();
		}

// USE /[MANUAL SF3 VIEWMODEL_APPLY AGENT]/

// USE /[MANUAL SF3 VIEWMODEL_DUPLICATE AGENT]/

// USE /[MANUAL SF3 VIEWMODEL_DESTROY AGENT]/
		public override void Destroy(string id)
		{
			Model = Models.Agent.Find(id, m_userContext, "FAGENT");
			if (Model == null)
				throw new ModelNotFoundException("Model not found");
			this.flashMessage = Model.Destroy();
		}

		/// <summary>
		/// Load selected row primary keys for all checklists
		/// </summary>
		public void LoadChecklistsSelectedIDs()
		{
		}

		protected override object GetViewModelValue(string identifier, object modelValue)
		{
			return identifier switch
			{
				"agent.photo" => ViewModelConversion.ToImage(modelValue),
				"agent.name" => ViewModelConversion.ToString(modelValue),
				"agent.birthdat" => ViewModelConversion.ToDateTime(modelValue),
				"agent.email" => ViewModelConversion.ToString(modelValue),
				"agent.telephon" => ViewModelConversion.ToString(modelValue),
				"agent.codagent" => ViewModelConversion.ToString(modelValue),
				_ => throw new Exception("Unexpected field identifier")
			};
		}

		#region Charts


		#endregion

		#region Custom code

// USE /[MANUAL SF3 VIEWMODEL_CUSTOM AGENT]/

		#endregion
	}
}
