﻿using JsonIgnoreAttribute = System.Text.Json.Serialization.JsonIgnoreAttribute;
using Microsoft.AspNetCore.Mvc.Rendering;
using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Specialized;
using System.ComponentModel.DataAnnotations;
using System.Data;
using System.Globalization;

using CSGenio.business;
using CSGenio.framework;
using CSGenio.persistence;
using GenioMVC.Helpers;
using GenioMVC.Models.Exception;
using GenioMVC.Models.Navigation;
using Quidgest.Persistence;
using Quidgest.Persistence.GenericQuery;

namespace GenioMVC.ViewModels.Prope
{
	public class Property_ViewModel : FormViewModel<Models.Prope>
	{
		[JsonIgnore]
		public override bool HasWriteConditions { get => false; }

		/// <summary>
		/// Reference for the Models MsqActive property
		/// </summary>
		[JsonIgnore]
		public bool MsqActive { get; set; } = false;

		#region Foreign keys

		/// <summary>
		/// Title: "Agent's name" | Type: "CE"
		/// </summary>
		public string ValCodagent { get; set; }

		#endregion

		/// <summary>
		/// Title: "Main photo" | Type: "IJ"
		/// </summary>
		[ImageThumbnailJsonConverter(30, 50)]
		public GenioMVC.ViewModels.ImageModel ValPhoto { get; set; }

		/// <summary>
		/// Title: "Title" | Type: "C"
		/// </summary>
		public string ValTitle { get; set; }

		/// <summary>
		/// Title: "Price" | Type: "$"
		/// </summary>
		public decimal? ValPrice { get; set; }

		/// <summary>
		/// Title: "Agent's name" | Type: "C"
		/// </summary>
		[ValidateSetAccess]
		public TableDBEdit<GenioMVC.Models.Agent> TableAgentName { get; set; }

		#region Navigations
		#endregion

		#region Auxiliar Keys for Image controls



		#endregion

		#region Extra database fields



		#endregion

		#region Fields for formulas


		#endregion


		public string ValCodprope { get; set; }

		/// <summary>
		/// FOR DESERIALIZATION ONLY
		/// A call to Init() needs to be made manually invoked after this constructor
		/// </summary>
		[Obsolete("For deserialization only")]
		public Property_ViewModel() : base(null!) { }

		public Property_ViewModel(UserContext userContext, bool nestedForm = false) : base(userContext, "FPROPERTY", nestedForm) { }

		public Property_ViewModel(UserContext userContext, Models.Prope row, bool nestedForm = false) : base(userContext, "FPROPERTY", row, nestedForm) { }

		public Property_ViewModel(UserContext userContext, string id, bool nestedForm = false, string[]? fieldsToLoad = null) : this(userContext, nestedForm)
		{
			this.Navigation.SetValue("prope", id);
			Model = Models.Prope.Find(id, userContext, "FPROPERTY", fieldsToQuery: fieldsToLoad);
			if (Model == null)
				throw new ModelNotFoundException("Model not found");
			InitModel();
		}

		protected override void InitLevels()
		{
			this.RoleToShow = CSGenio.framework.Role.ROLE_1;
			this.RoleToEdit = CSGenio.framework.Role.ROLE_1;
		}

		#region Form conditions

		public override StatusMessage InsertConditions()
		{
			return InsertConditions(m_userContext);
		}

		public static StatusMessage InsertConditions(UserContext userContext)
		{
			var m_userContext = userContext;
			StatusMessage result = new StatusMessage(Status.OK, "");
			Models.Prope model = new Models.Prope(userContext) { Identifier = "FPROPERTY" };

			var navigation = m_userContext.CurrentNavigation;
			// The "LoadKeysFormHistory" must be after the "LoadEPH" because the PHE's in the tree mark Foreign Keys to null
			// (since they cannot assign multiple values to a single field) and thus the value that comes from Navigation is lost.
			// And this makes it more like the order of loading the model when opening the form.
			model.LoadEPH("FPROPERTY");
			if (navigation != null)
				model.LoadKeysFormHistory(navigation, navigation.CurrentLevel.Level);

			var tableResult = model.EvaluateTableConditions(ConditionType.INSERT);
			result.MergeStatusMessage(tableResult);
			return result;
		}

		public override StatusMessage UpdateConditions()
		{
			StatusMessage result = new StatusMessage(Status.OK, "");
			var model = Model;

			var tableResult = model.EvaluateTableConditions(ConditionType.UPDATE);
			result.MergeStatusMessage(tableResult);
			return result;
		}

		public override StatusMessage DeleteConditions()
		{
			StatusMessage result = new StatusMessage(Status.OK, "");
			var model = Model;

			var tableResult = model.EvaluateTableConditions(ConditionType.DELETE);
			result.MergeStatusMessage(tableResult);
			return result;
		}

		public override StatusMessage ViewConditions()
		{
			var model = Model;
			StatusMessage result = model.EvaluateTableConditions(ConditionType.VIEW);
			var tableResult = model.EvaluateTableConditions(ConditionType.VIEW);
			result.MergeStatusMessage(tableResult);
			return result;
		}

		protected override StatusMessage EvaluateWriteConditions(bool isApply)
		{
			Models.Prope model = Model;
			StatusMessage result = new StatusMessage(Status.OK, "");
			return result;
		}

		public StatusMessage EvaluateTableConditions(ConditionType type)
		{
			return Model.EvaluateTableConditions(type);
		}

		#endregion

		#region Mapper

		public override void MapFromModel(Models.Prope m)
		{
			if (m == null)
			{
				CSGenio.framework.Log.Error("Map Model (Prope) to ViewModel (Property) - Model is a null reference");
				throw new ModelNotFoundException("Model not found");
			}

			try
			{
				ValCodagent = ViewModelConversion.ToString(m.ValCodagent);
				ValPhoto = ViewModelConversion.ToImage(m.ValPhoto);
				ValTitle = ViewModelConversion.ToString(m.ValTitle);
				ValPrice = ViewModelConversion.ToNumeric(m.ValPrice);
				ValCodprope = ViewModelConversion.ToString(m.ValCodprope);
			}
			catch (Exception)
			{
				CSGenio.framework.Log.Error("Map Model (Prope) to ViewModel (Property) - Error during mapping");
				throw;
			}
		}

		/// <summary>
		/// Performs the mapping of field values from the ViewModel to the Model.
		/// </summary>
		/// <exception cref="ModelNotFoundException">Thrown if <paramref name="m"/> is null.</exception>
		public override void MapToModel()
		{
			MapToModel(this.Model);
		}

		/// <summary>
		/// Performs the mapping of field values from the ViewModel to the Model.
		/// </summary>
		/// <param name="m">The Model to be filled.</param>
		/// <exception cref="ModelNotFoundException">Thrown if <paramref name="m"/> is null.</exception>
		public override void MapToModel(Models.Prope m)
		{
			if (m == null)
			{
				CSGenio.framework.Log.Error("Map ViewModel (Property) to Model (Prope) - Model is a null reference");
				throw new ModelNotFoundException("Model not found");
			}

			try
			{
				m.ValCodagent = ViewModelConversion.ToString(ValCodagent);
				m.ValPhoto = ViewModelConversion.ToImage(ValPhoto);
				m.ValTitle = ViewModelConversion.ToString(ValTitle);
				m.ValPrice = ViewModelConversion.ToNumeric(ValPrice);
				m.ValCodprope = ViewModelConversion.ToString(ValCodprope);
			}
			catch (Exception)
			{
				CSGenio.framework.Log.Error($"Map ViewModel (Property) to Model (Prope) - Error during mapping. All user values: {HasDisabledUserValuesSecurity}");
				throw;
			}
		}

		/// <summary>
		/// Sets the value of a single property of the view model based on the provided table and field names.
		/// </summary>
		/// <param name="fullFieldName">The full field name in the format "table.field".</param>
		/// <param name="value">The field value.</param>
		/// <exception cref="ArgumentNullException">Thrown if <paramref name="fullFieldName"/> is null.</exception>
		public override void SetViewModelValue(string fullFieldName, object value)
		{
			try
			{
				ArgumentNullException.ThrowIfNull(fullFieldName);
				// Obtain a valid value from JsonValueKind that can come from "prefillValues" during the pre-filling of fields during insertion
				var _value = ViewModelConversion.ToRawValue(value);

				switch(fullFieldName)
				{
					case "prope.codagent":
						this.ValCodagent = ViewModelConversion.ToString(_value);
						break;
					case "prope.title":
						this.ValTitle = ViewModelConversion.ToString(_value);
						break;
					case "prope.price":
						this.ValPrice = ViewModelConversion.ToNumeric(_value);
						break;
					case "prope.codprope":
						this.ValCodprope = ViewModelConversion.ToString(_value);
						break;
					default:
						Log.Error($"SetViewModelValue (Property) - Unexpected field identifier {fullFieldName}");
						break;
				}
			}
			catch (Exception ex)
			{
				throw new FrameworkException(Resources.Resources.PEDIMOS_DESCULPA__OC63848, "SetViewModelValue (Property)", "Unexpected error", ex);
			}
		}

		#endregion

		/// <summary>
		/// Reads the Model from the database based on the key that is in the history or that was passed through the parameter
		/// </summary>
		/// <param name="id">The primary key of the record that needs to be read from the database. Leave NULL to use the value from the History.</param>
		public override void LoadModel(string id = null)
		{
			try { Model = Models.Prope.Find(id ?? Navigation.GetStrValue("prope"), m_userContext, "FPROPERTY"); }
			finally { Model ??= new Models.Prope(m_userContext) { Identifier = "FPROPERTY" }; }

			base.LoadModel();
		}


		public override void Load(NameValueCollection qs, bool editable, bool ajaxRequest = false, bool lazyLoad = false)
		{
			this.editable = editable;
			CSGenio.business.Area oldvalues = null;

			// TODO: Deve ser substituido por search do CSGenioA
			try
			{
				Model = Models.Prope.Find(Navigation.GetStrValue("prope"), m_userContext, "FPROPERTY");
			}
			finally
			{
				if (Navigation.CurrentLevel.FormMode == FormMode.New || Navigation.CurrentLevel.FormMode == FormMode.Duplicate)
				{
					if (Model == null)
						throw new ModelNotFoundException("Model not found");

					LoadDefaultValues();
				}
				else
				{
					if (Model == null)
						throw new ModelNotFoundException("Model not found");

					oldvalues = Model.klass;
				}
			}

			Model.Identifier = "FPROPERTY";
			InitModel(qs, lazyLoad);

			if (Navigation.CurrentLevel.FormMode == FormMode.New || Navigation.CurrentLevel.FormMode == FormMode.Edit || Navigation.CurrentLevel.FormMode == FormMode.Duplicate)
			{
				// MH - Voltar calcular as formulas to "atualizar" os Qvalues dos fields fixos
				// Conexão deve estar aberta de fora. Podem haver formulas que utilizam funções "manuais".
				MapToModel(Model);
				// Preencher operações internas
				Model.klass.fillInternalOperations(m_userContext.PersistentSupport, oldvalues);
				MapFromModel(Model);
			}

			// Load just the selected row primary keys for checklists.
			// Needed for submitting forms incase checklists are in collapsible zones that have not been expanded to load the checklist data.
			LoadChecklistsSelectedIDs();
		}

		protected override void FillExtraProperties()
		{
		}

		protected override void LoadDocumentsProperties(Models.Prope row)
		{
		}

		/// <summary>
		/// Load Partial
		/// </summary>
		/// <param name="lazyLoad">Lazy loading of dropdown items</param>
		public override void LoadPartial(NameValueCollection qs, bool lazyLoad = false)
		{
			// MH [bugfix] - Quando o POST da ficha falha, ao recaregar a view os documentos na BD perdem alguma informação (ex: name do file)
			if (Model == null)
			{
				// Precisamos fazer o Find to obter as chaves dos documentos que já foram anexados
				// TODO: Conseguir passar estas chaves no POST to poder retirar o Find.
				Model = Models.Prope.Find(Navigation.GetStrValue("prope"), m_userContext, "FPROPERTY");
				if (Model == null)
				{
					Model = new Models.Prope(m_userContext) { Identifier = "FPROPERTY" };
					Model.klass.QPrimaryKey = Navigation.GetStrValue("prope");
				}
				MapToModel(Model);
				LoadDocumentsProperties(Model);
			}
			// Add characteristics
			Characs = new List<string>();

			Load_Propertyagentname____(qs, lazyLoad);
// USE /[MANUAL SF3 VIEWMODEL_LOADPARTIAL PROPERTY]/
		}

// USE /[MANUAL SF3 VIEWMODEL_NEW PROPERTY]/

		// Preencher Qvalues default dos fields do form
		protected override void LoadDefaultValues()
		{
		}

		public override CrudViewModelValidationResult Validate()
		{
			CrudViewModelFieldValidator validator = new(m_userContext.User.Language);


			validator.StringLength("ValTitle", Resources.Resources.TITLE21885, ValTitle, 50);

			return validator.GetResult();
		}

// USE /[MANUAL SF3 VIEWMODEL_SAVE PROPERTY]/
		public override void Save()
		{


			base.Save();
		}

// USE /[MANUAL SF3 VIEWMODEL_APPLY PROPERTY]/

// USE /[MANUAL SF3 VIEWMODEL_DUPLICATE PROPERTY]/

// USE /[MANUAL SF3 VIEWMODEL_DESTROY PROPERTY]/
		public override void Destroy(string id)
		{
			Model = Models.Prope.Find(id, m_userContext, "FPROPERTY");
			if (Model == null)
				throw new ModelNotFoundException("Model not found");
			this.flashMessage = Model.Destroy();
		}

		/// <summary>
		/// Load selected row primary keys for all checklists
		/// </summary>
		public void LoadChecklistsSelectedIDs()
		{
		}

		/// <summary>
		/// TableAgentName -> (DB)
		/// </summary>
		/// <param name="qs"></param>
		/// <param name="lazyLoad">Lazy loading of dropdown items</param>
		public void Load_Propertyagentname____(NameValueCollection qs, bool lazyLoad = false)
		{
			bool propertyagentname____DoLoad = true;
			CriteriaSet propertyagentname____Conds = CriteriaSet.And();
			{
				object hValue = Navigation.GetValue("agent", true);
				if (hValue != null && !(hValue is Array) && !string.IsNullOrEmpty(Convert.ToString(hValue)))
				{
					propertyagentname____Conds.Equal(CSGenioAagent.FldCodagent, Navigation.GetValue("agent"));
					this.ValCodagent = Navigation.GetStrValue("agent");
				}
			}

			TableAgentName = new TableDBEdit<Models.Agent>
			{
				IsLazyLoad = lazyLoad
			};

			if (lazyLoad)
			{
				if (Navigation.CurrentLevel.GetEntry("RETURN_agent") != null)
				{
					this.ValCodagent = Navigation.GetStrValue("RETURN_agent");
					Navigation.CurrentLevel.SetEntry("RETURN_agent", null);
				}
				FillDependant_PropertyTableAgentName(lazyLoad);
				//Check if foreignkey comes from history
				TableAgentName.FilledByHistory = Navigation.CheckFilledByHistory("agent");
				return;
			}

			if (propertyagentname____DoLoad)
			{
				List<ColumnSort> sorts = new List<ColumnSort>();
				ColumnSort requestedSort = GetRequestSort(TableAgentName, "sTableAgentName", "dTableAgentName", qs, "agent");
				if (requestedSort != null)
					sorts.Add(requestedSort);

				string query = "";
				if (!string.IsNullOrEmpty(qs["TableAgentName_tableFilters"]))
					TableAgentName.TableFilters = bool.Parse(qs["TableAgentName_tableFilters"]);
				else
					TableAgentName.TableFilters = false;

				query = qs["qTableAgentName"];

				//RS 26.07.2016 O preenchimento da lista de ajuda dos Dbedits passa a basear-se apenas no campo do próprio DbEdit
				// O interface de pesquisa rápida não fica coerente quando se visualiza apenas uma coluna mas a pesquisa faz matching com 5 ou 6 colunas diferentes
				//  tornando confuso to o user porque determinada row foi devolvida quando o Qresult não mostra como o matching foi feito
				CriteriaSet search_filters = CriteriaSet.And();
				if (!string.IsNullOrEmpty(query))
				{
					search_filters.Like(CSGenioAagent.FldName, query + "%");
				}
				propertyagentname____Conds.SubSet(search_filters);

				string tryParsePage = qs["pTableAgentName"] != null ? qs["pTableAgentName"].ToString() : "1";
				int page = !string.IsNullOrEmpty(tryParsePage) ? int.Parse(tryParsePage) : 1;
				int numberItems = CSGenio.framework.Configuration.NrRegDBedit;
				int offset = (page - 1) * numberItems;

				FieldRef[] fields = new FieldRef[] { CSGenioAagent.FldCodagent, CSGenioAagent.FldName, CSGenioAagent.FldEmail, CSGenioAagent.FldZzstate };

// USE /[MANUAL SF3 OVERRQ PROPERTY_AGENTNAME]/

				// Limitation by Zzstate
				/*
					Records that are currently being inserted or duplicated will also be included.
					Client-side persistence will try to fill the "text" value of that option.
				*/
				if (Navigation.checkFormMode("agent", FormMode.New) || Navigation.checkFormMode("agent", FormMode.Duplicate))
					propertyagentname____Conds.SubSet(CriteriaSet.Or()
						.Equal(CSGenioAagent.FldZzstate, 0)
						.Equal(CSGenioAagent.FldCodagent, Navigation.GetStrValue("agent")));
				else
					propertyagentname____Conds.Criterias.Add(new Criteria(new ColumnReference(CSGenioAagent.FldZzstate), CriteriaOperator.Equal, 0));

				FieldRef firstVisibleColumn = new FieldRef("agent", "name");
				ListingMVC<CSGenioAagent> listing = Models.ModelBase.Where<CSGenioAagent>(m_userContext, false, propertyagentname____Conds, fields, offset, numberItems, sorts, "LED_PROPERTYAGENTNAME____", true, false, firstVisibleColumn: firstVisibleColumn);

				TableAgentName.SetPagination(page, numberItems, listing.HasMore, listing.GetTotal, listing.TotalRecords);
				TableAgentName.Query = query;
				TableAgentName.Elements = listing.RowsForViewModel<GenioMVC.Models.Agent>((r) => new GenioMVC.Models.Agent(m_userContext, r, true, _fieldsToSerialize_PROPERTYAGENTNAME____));

				//created by [ MH ] at [ 14.04.2016 ] - Foi alterada a forma de retornar a key do novo registo inserido / editado no form de apoio do DBEdit.
				//last update by [ MH ] at [ 10.05.2016 ] - Validação se key encontra-se no level atual, as chaves dos niveis anteriores devem ser ignorados.
				if (Navigation.CurrentLevel.GetEntry("RETURN_agent") != null)
				{
					this.ValCodagent = Navigation.GetStrValue("RETURN_agent");
					Navigation.CurrentLevel.SetEntry("RETURN_agent", null);
				}

				TableAgentName.List = new SelectList(TableAgentName.Elements.ToSelectList(x => x.ValName, x => x.ValCodagent,  x => x.ValCodagent == this.ValCodagent), "Value", "Text", this.ValCodagent);
				FillDependant_PropertyTableAgentName();

				//Check if foreignkey comes from history
				TableAgentName.FilledByHistory = Navigation.CheckFilledByHistory("agent");
			}
		}

		/// <summary>
		/// Get Dependant fields values -> TableAgentName (DB)
		/// </summary>
		/// <param name="PKey">Primary Key of Agent</param>
		public ConcurrentDictionary<string, object> GetDependant_PropertyTableAgentName(string PKey)
		{
			FieldRef[] refDependantFields = [CSGenioAagent.FldCodagent, CSGenioAagent.FldName];

			var returnEmptyDependants = false;
			CriteriaSet wherecodition = CriteriaSet.And();

			// Return default values
			if (GlobalFunctions.emptyG(PKey) == 1)
				returnEmptyDependants = true;

			// Check if the limit(s) is filled if exists
			// - - - - - - - - - - - - - - - - - - - - -

			if (returnEmptyDependants)
				return GetViewModelFieldValues(refDependantFields);

			PersistentSupport sp = m_userContext.PersistentSupport;
			User u = m_userContext.User;

			CSGenioAagent tempArea = new(u);

			// Fields to select
			SelectQuery querySelect = new();
			querySelect.PageSize(1);
			foreach (FieldRef field in refDependantFields)
				querySelect.Select(field);

			querySelect.From(tempArea.QSystem, tempArea.TableName, tempArea.Alias)
				.Where(wherecodition.Equal(CSGenioAagent.FldCodagent, PKey));

			string[] dependantFields = refDependantFields.Select(f => f.FullName).ToArray();
			QueryUtils.SetInnerJoins(dependantFields, null, tempArea, querySelect);

			ArrayList values = sp.executeReaderOneRow(querySelect);
			bool useDefaults = values.Count == 0;

			if (useDefaults)
				return GetViewModelFieldValues(refDependantFields);
			return GetViewModelFieldValues(refDependantFields, values);
		}

		/// <summary>
		/// Fill Dependant fields values -> TableAgentName (DB)
		/// </summary>
		/// <param name="lazyLoad">Lazy loading of dropdown items</param>
		public void FillDependant_PropertyTableAgentName(bool lazyLoad = false)
		{
			var row = GetDependant_PropertyTableAgentName(this.ValCodagent);
			try
			{

				// Fill List fields
				this.ValCodagent = ViewModelConversion.ToString(row["agent.codagent"]);
				TableAgentName.Value = (string)row["agent.name"];
				if (GlobalFunctions.emptyG(this.ValCodagent) == 1)
				{
					this.ValCodagent = "";
					TableAgentName.Value = "";
					Navigation.ClearValue("agent");
				}
				else if (lazyLoad)
				{
					TableAgentName.SetPagination(1, 0, false, false, 1);
					TableAgentName.List = new SelectList(new List<SelectListItem>()
					{
						new SelectListItem
						{
							Value = Convert.ToString(this.ValCodagent),
							Text = Convert.ToString(TableAgentName.Value),
							Selected = true
						}
					}, "Value", "Text", this.ValCodagent);
				}

				TableAgentName.Selected = this.ValCodagent;
			}
			catch (Exception ex)
			{
				CSGenio.framework.Log.Error(string.Format("FillDependant_Error (TableAgentName): {0}; {1}", ex.Message, ex.InnerException != null ? ex.InnerException.Message : ""));
			}
		}

		private readonly string[] _fieldsToSerialize_PROPERTYAGENTNAME____ = ["Agent", "Agent.ValCodagent", "Agent.ValZzstate", "Agent.ValName", "Agent.ValEmail"];

		protected override object GetViewModelValue(string identifier, object modelValue)
		{
			return identifier switch
			{
				"prope.codagent" => ViewModelConversion.ToString(modelValue),
				"prope.photo" => ViewModelConversion.ToImage(modelValue),
				"prope.title" => ViewModelConversion.ToString(modelValue),
				"prope.price" => ViewModelConversion.ToNumeric(modelValue),
				"prope.codprope" => ViewModelConversion.ToString(modelValue),
				"agent.codagent" => ViewModelConversion.ToString(modelValue),
				"agent.name" => ViewModelConversion.ToString(modelValue),
				_ => throw new Exception("Unexpected field identifier")
			};
		}

		#region Charts


		#endregion

		#region Custom code

// USE /[MANUAL SF3 VIEWMODEL_CUSTOM PROPERTY]/

		#endregion
	}
}
