using ExecuteQueryCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Web;
using CSGenio.persistence;
using CSGenio.business;
using CSGenio.framework;
using Quidgest.Persistence.GenericQuery;
using Quidgest.Persistence;

namespace CSGenio.business
{
    public class ReindexFunctions
    {
        public PersistentSupport sp { get; set; }
        public User user { get; set; }
        public bool Zero { get; set; }

        public ReindexFunctions(PersistentSupport sp, User user, bool Zero = false) {
            this.sp = sp;
            this.user = user;
            this.Zero = Zero;
        }   

        public void DeleteInvalidRows(CancellationToken cToken) {
            List<int> zzstateToRemove = new List<int> { 1, 11 };
            DataMatrix dm;
            sp.openConnection();

            /* --- SF3AGENT --- */
            dm = sp.Execute(
                new SelectQuery()
                .Select(CSGenioAagent.FldCodagent)
                .From(CSGenioAagent.AreaAGENT)
                .Where(CriteriaSet.And().In(CSGenioAagent.FldZzstate, zzstateToRemove))
                );

            for (int i = 0; i < dm.NumRows; i++)
            {
                CSGenioAagent model = new CSGenioAagent(user);
                model.ValCodagent = dm.GetKey(i, 0);

                try
                {
                    model.delete(sp);
                }
                //Not every exception should be allowed to continue record deletion, only business exceptions need to be caught and allow to deletion continue.
                //If there are other types of exceptions, such as database connection problems, for example, execution should be stopped immediately
                catch(BusinessException ex)
                {
                    Log.Error((ex.UserMessage != null) ? ex.UserMessage : ex.Message);
                }
            }
                

            /* --- SF3MEM --- */
            dm = sp.Execute(
                new SelectQuery()
                .Select(CSGenioAmem.FldCodmem)
                .From(CSGenioAmem.AreaMEM)
                .Where(CriteriaSet.And().In(CSGenioAmem.FldZzstate, zzstateToRemove))
                );

            for (int i = 0; i < dm.NumRows; i++)
            {
                CSGenioAmem model = new CSGenioAmem(user);
                model.ValCodmem = dm.GetKey(i, 0);

                try
                {
                    model.delete(sp);
                }
                //Not every exception should be allowed to continue record deletion, only business exceptions need to be caught and allow to deletion continue.
                //If there are other types of exceptions, such as database connection problems, for example, execution should be stopped immediately
                catch(BusinessException ex)
                {
                    Log.Error((ex.UserMessage != null) ? ex.UserMessage : ex.Message);
                }
            }
                

            /* --- UserLogin --- */
            dm = sp.Execute(
                new SelectQuery()
                .Select(CSGenioApsw.FldCodpsw)
                .From(CSGenioApsw.AreaPSW)
                .Where(CriteriaSet.And().In(CSGenioApsw.FldZzstate, zzstateToRemove))
                );

            for (int i = 0; i < dm.NumRows; i++)
            {
                CSGenioApsw model = new CSGenioApsw(user);
                model.ValCodpsw = dm.GetKey(i, 0);

                try
                {
                    model.delete(sp);
                }
                //Not every exception should be allowed to continue record deletion, only business exceptions need to be caught and allow to deletion continue.
                //If there are other types of exceptions, such as database connection problems, for example, execution should be stopped immediately
                catch(BusinessException ex)
                {
                    Log.Error((ex.UserMessage != null) ? ex.UserMessage : ex.Message);
                }
            }
                

            /* --- AsyncProcess --- */
            dm = sp.Execute(
                new SelectQuery()
                .Select(CSGenioAs_apr.FldCodascpr)
                .From(CSGenioAs_apr.AreaS_APR)
                .Where(CriteriaSet.And().In(CSGenioAs_apr.FldZzstate, zzstateToRemove))
                );

            for (int i = 0; i < dm.NumRows; i++)
            {
                CSGenioAs_apr model = new CSGenioAs_apr(user);
                model.ValCodascpr = dm.GetKey(i, 0);

                try
                {
                    model.delete(sp);
                }
                //Not every exception should be allowed to continue record deletion, only business exceptions need to be caught and allow to deletion continue.
                //If there are other types of exceptions, such as database connection problems, for example, execution should be stopped immediately
                catch(BusinessException ex)
                {
                    Log.Error((ex.UserMessage != null) ? ex.UserMessage : ex.Message);
                }
            }
                

            /* --- NotificationEmailSignature --- */
            dm = sp.Execute(
                new SelectQuery()
                .Select(CSGenioAs_nes.FldCodsigna)
                .From(CSGenioAs_nes.AreaS_NES)
                .Where(CriteriaSet.And().In(CSGenioAs_nes.FldZzstate, zzstateToRemove))
                );

            for (int i = 0; i < dm.NumRows; i++)
            {
                CSGenioAs_nes model = new CSGenioAs_nes(user);
                model.ValCodsigna = dm.GetKey(i, 0);

                try
                {
                    model.delete(sp);
                }
                //Not every exception should be allowed to continue record deletion, only business exceptions need to be caught and allow to deletion continue.
                //If there are other types of exceptions, such as database connection problems, for example, execution should be stopped immediately
                catch(BusinessException ex)
                {
                    Log.Error((ex.UserMessage != null) ? ex.UserMessage : ex.Message);
                }
            }
                

            /* --- NotificationMessage --- */
            dm = sp.Execute(
                new SelectQuery()
                .Select(CSGenioAs_nm.FldCodmesgs)
                .From(CSGenioAs_nm.AreaS_NM)
                .Where(CriteriaSet.And().In(CSGenioAs_nm.FldZzstate, zzstateToRemove))
                );

            for (int i = 0; i < dm.NumRows; i++)
            {
                CSGenioAs_nm model = new CSGenioAs_nm(user);
                model.ValCodmesgs = dm.GetKey(i, 0);

                try
                {
                    model.delete(sp);
                }
                //Not every exception should be allowed to continue record deletion, only business exceptions need to be caught and allow to deletion continue.
                //If there are other types of exceptions, such as database connection problems, for example, execution should be stopped immediately
                catch(BusinessException ex)
                {
                    Log.Error((ex.UserMessage != null) ? ex.UserMessage : ex.Message);
                }
            }
                

            /* --- SF3PROPE --- */
            dm = sp.Execute(
                new SelectQuery()
                .Select(CSGenioAprope.FldCodprope)
                .From(CSGenioAprope.AreaPROPE)
                .Where(CriteriaSet.And().In(CSGenioAprope.FldZzstate, zzstateToRemove))
                );

            for (int i = 0; i < dm.NumRows; i++)
            {
                CSGenioAprope model = new CSGenioAprope(user);
                model.ValCodprope = dm.GetKey(i, 0);

                try
                {
                    model.delete(sp);
                }
                //Not every exception should be allowed to continue record deletion, only business exceptions need to be caught and allow to deletion continue.
                //If there are other types of exceptions, such as database connection problems, for example, execution should be stopped immediately
                catch(BusinessException ex)
                {
                    Log.Error((ex.UserMessage != null) ? ex.UserMessage : ex.Message);
                }
            }
                

            /* --- AsyncProcessArgument --- */
            dm = sp.Execute(
                new SelectQuery()
                .Select(CSGenioAs_arg.FldCodargpr)
                .From(CSGenioAs_arg.AreaS_ARG)
                .Where(CriteriaSet.And().In(CSGenioAs_arg.FldZzstate, zzstateToRemove))
                );

            for (int i = 0; i < dm.NumRows; i++)
            {
                CSGenioAs_arg model = new CSGenioAs_arg(user);
                model.ValCodargpr = dm.GetKey(i, 0);

                try
                {
                    model.delete(sp);
                }
                //Not every exception should be allowed to continue record deletion, only business exceptions need to be caught and allow to deletion continue.
                //If there are other types of exceptions, such as database connection problems, for example, execution should be stopped immediately
                catch(BusinessException ex)
                {
                    Log.Error((ex.UserMessage != null) ? ex.UserMessage : ex.Message);
                }
            }
                

            /* --- AsyncProcessAttachments --- */
            dm = sp.Execute(
                new SelectQuery()
                .Select(CSGenioAs_pax.FldCodpranx)
                .From(CSGenioAs_pax.AreaS_PAX)
                .Where(CriteriaSet.And().In(CSGenioAs_pax.FldZzstate, zzstateToRemove))
                );

            for (int i = 0; i < dm.NumRows; i++)
            {
                CSGenioAs_pax model = new CSGenioAs_pax(user);
                model.ValCodpranx = dm.GetKey(i, 0);

                try
                {
                    model.delete(sp);
                }
                //Not every exception should be allowed to continue record deletion, only business exceptions need to be caught and allow to deletion continue.
                //If there are other types of exceptions, such as database connection problems, for example, execution should be stopped immediately
                catch(BusinessException ex)
                {
                    Log.Error((ex.UserMessage != null) ? ex.UserMessage : ex.Message);
                }
            }
                

            /* --- UserAuthorization --- */
            dm = sp.Execute(
                new SelectQuery()
                .Select(CSGenioAs_ua.FldCodua)
                .From(CSGenioAs_ua.AreaS_UA)
                .Where(CriteriaSet.And().In(CSGenioAs_ua.FldZzstate, zzstateToRemove))
                );

            for (int i = 0; i < dm.NumRows; i++)
            {
                CSGenioAs_ua model = new CSGenioAs_ua(user);
                model.ValCodua = dm.GetKey(i, 0);

                try
                {
                    model.delete(sp);
                }
                //Not every exception should be allowed to continue record deletion, only business exceptions need to be caught and allow to deletion continue.
                //If there are other types of exceptions, such as database connection problems, for example, execution should be stopped immediately
                catch(BusinessException ex)
                {
                    Log.Error((ex.UserMessage != null) ? ex.UserMessage : ex.Message);
                }
            }
                
            
            //Hard Coded Tabels
            //These can be directly removed

            /* --- SF3mem --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3mem")
                .Where(CriteriaSet.And().In("SF3mem", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3cfg --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3cfg")
                .Where(CriteriaSet.And().In("SF3cfg", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3lstusr --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3lstusr")
                .Where(CriteriaSet.And().In("SF3lstusr", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3lstcol --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3lstcol")
                .Where(CriteriaSet.And().In("SF3lstcol", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3lstren --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3lstren")
                .Where(CriteriaSet.And().In("SF3lstren", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3usrwid --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3usrwid")
                .Where(CriteriaSet.And().In("SF3usrwid", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3usrcfg --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3usrcfg")
                .Where(CriteriaSet.And().In("SF3usrcfg", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3usrset --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3usrset")
                .Where(CriteriaSet.And().In("SF3usrset", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3wkfact --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3wkfact")
                .Where(CriteriaSet.And().In("SF3wkfact", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3wkfcon --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3wkfcon")
                .Where(CriteriaSet.And().In("SF3wkfcon", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3wkflig --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3wkflig")
                .Where(CriteriaSet.And().In("SF3wkflig", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3wkflow --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3wkflow")
                .Where(CriteriaSet.And().In("SF3wkflow", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3notifi --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3notifi")
                .Where(CriteriaSet.And().In("SF3notifi", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3prmfrm --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3prmfrm")
                .Where(CriteriaSet.And().In("SF3prmfrm", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3scrcrd --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3scrcrd")
                .Where(CriteriaSet.And().In("SF3scrcrd", "ZZSTATE", zzstateToRemove)));
                
            /* --- docums --- */
            sp.Execute(new DeleteQuery()
                .Delete("docums")
                .Where(CriteriaSet.And().In("docums", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3postit --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3postit")
                .Where(CriteriaSet.And().In("SF3postit", "ZZSTATE", zzstateToRemove)));
                
            /* --- hashcd --- */
            sp.Execute(new DeleteQuery()
                .Delete("hashcd")
                .Where(CriteriaSet.And().In("hashcd", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3alerta --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3alerta")
                .Where(CriteriaSet.And().In("SF3alerta", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3altent --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3altent")
                .Where(CriteriaSet.And().In("SF3altent", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3talert --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3talert")
                .Where(CriteriaSet.And().In("SF3talert", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3delega --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3delega")
                .Where(CriteriaSet.And().In("SF3delega", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3TABDINAMIC --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3TABDINAMIC")
                .Where(CriteriaSet.And().In("SF3TABDINAMIC", "ZZSTATE", zzstateToRemove)));
                
            /* --- UserAuthorization --- */
            sp.Execute(new DeleteQuery()
                .Delete("UserAuthorization")
                .Where(CriteriaSet.And().In("UserAuthorization", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3altran --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3altran")
                .Where(CriteriaSet.And().In("SF3altran", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3workflowtask --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3workflowtask")
                .Where(CriteriaSet.And().In("SF3workflowtask", "ZZSTATE", zzstateToRemove)));
                
            /* --- SF3workflowprocess --- */
            sp.Execute(new DeleteQuery()
                .Delete("SF3workflowprocess")
                .Where(CriteriaSet.And().In("SF3workflowprocess", "ZZSTATE", zzstateToRemove)));
                

            sp.closeConnection();
        }




 
        // USE /[MANUAL RDX_STEP]/
    }
}