using System;
using CSGenio.business;
using System.Collections.Generic;
using CSGenio.framework;
using System.Text;
using CSGenio.persistence;
//using System.Web.UI;
using System.IO;
using Quidgest.Persistence;
using Quidgest.Persistence.GenericQuery;
using System.Xml;

namespace CSGenio.business
{

    //classe que representa o horrio de trabalho recebido do flash
    //Exemplo de horario recebido
    /*  <HORTRABALHO>
        <GRAF INICIO = "09:3013:0012:0419:3113:1809:3013:20"
        FIM = "12:0017:3013:1817:3023:5923:59"
        LISTA = "1122344"
        MOTIVO = "03 ASS. FAM.Men. 10"
        TLMOTIVO = "01 DISPENSA02 INT. CAFɧ03 ASS. FAM.Men. 1004 FRIAS05 1/2  FRIAS06 SERV. EXTERNO07 MATERNIDADE08 AMAMENTAO09 DOENA  &lt; 60 D10 DOENA &gt; 60 D11 INTERNAMENTO12 ASS. FAMILIA13 TRAB. ESTUDANTE14 INJUSTIF15 LI. S/V 90 D16 LI. S/V 1 ANO17 LI. S/V LD18 ATR. TRANS.19 NASCIMENTO20 GRAVIDEZ RISCO21 ACID. SERV.22 T. AMBULATRIO23 ART. 66 1/224 ART. 6625 DADOR SANGUE26 SAIDA JUDICIAL27 JUSTIFICADO28 ALMOO NATAL29 GREVE TRANS.30 JUNTA MDICA31 ALMOO32 FESTA NATAL33 ELEIES34 AVARIA TRANS.35 GREVE36 FALTA INJUS. 1/237 FALTA INJUS.38 PROVAS/CONCURSO39 MD/CASLISBOA40 GUIA MARCHA41 FUNERAL42 ALMOO/SERVIO43 ESTGIO44 PEREGRINAO45 PATERNIDADE46 DESBARATIZAO47 LIC. PARENTAL48 COMPENSAO49 FORMAO50 NOJO51 CASAMENTO52 SEGURO53 TOL. PONTO54 REQUISIO"/>
        CHORARIO = S
        CPICAGEM = S
        CPERIODO S
        CATRASO = S
        OBSERVAC = observacoes
        </HORTRABALHO>*/

    public class WorkSchedule
    {
        public List<string> start = new List<string>();//start de cada Qvalue
        public List<string> fim = new List<string>();//fim de cada Qvalue
        public List<string> lista = new List<string>();//lista de tipos de Qvalues
        public List<string> motivo = new List<string>();//lista de motivos de atraso
        public List<string> lista_motivos = new List<string>();//lista de motivos de atrasos geral
        public bool horarioConsulta;//se o horrio  de consulta, i.e no pode ser alterado
        public bool picagemConsulta;//se as picagens so de consulta, i.e no podem ser alteradas 
        public bool periodoConsulta;//se os periodos de presena so de consulta, i.e no podem ser alterados
        public bool atrasoConsulta;//se os atrasos so s de consulta, i.e no podem ser alterados
        public enum RegistrationType { horario = 1, periodo = 2, atraso = 3, picagem = 4 };//tipo de registo (adicionado  varivel lista)
        public static char separador = '';//separador utilizado na comunicao com o flash
        public string isDirigente;//se  dirigente
        public string observacoes; //se so observacoes

        /// <summary>
        /// Constructor da classe
        /// </summary>
        /// <param name="consultaHorario">se o horrio  de consulta, i.e no pode ser alterado</param>
        /// <param name="consultaPicagem">se as picagens so de consulta, i.e no podem ser alteradas</param>
        /// <param name="consultaPeriodo">se os periodos de presena so de consulta, i.e no podem ser alterados</param>
        /// <param name="consultaAtraso">se os atrasos so s de consulta, i.e no podem ser alterados</param>
        public WorkSchedule(bool consultaHorario, bool consultaPicagem, bool consultaPeriodo, bool consultaAtraso, string dirigente)
        {
            horarioConsulta = consultaHorario;
            picagemConsulta = consultaPicagem;
            periodoConsulta = consultaPeriodo;
            atrasoConsulta = consultaAtraso;
            isDirigente = dirigente;

        }
       
        /// <summary>
        /// Mtodo que constroi o xml a enviar to o flash
        /// </summary>
        /// <returns></returns>
        public string buildXML()
        {
            XmlDocument xmlDoc = new XmlDocument();
            XmlElement rootNode = xmlDoc.CreateElement("HORTRABALHO");

            XmlElement graf = xmlDoc.CreateElement("GRAF");

            string valInicio = "";
            foreach (string valorInicio in start)
            {
                if (!string.IsNullOrEmpty(valInicio))
                    valInicio += separador;
               
                valInicio += valorInicio;
            }
            graf.SetAttribute("INICIO", valInicio);
            string valFim = "";
            foreach (string valorFim in fim)
            {
                if (!string.IsNullOrEmpty(valFim))
                    valFim += separador;                
                
				valFim += valorFim;
            }
            graf.SetAttribute("FIM", valFim);
            string valLista = "";
            foreach (string valorLista in lista)
            {
                if (!string.IsNullOrEmpty(valLista))
                    valLista += separador;
                
                valLista += valorLista;
            }
            graf.SetAttribute("LISTA", valLista);
            string valMotivo = separador.ToString();
            foreach (string valorMotivo in motivo)
            {
                valMotivo += valorMotivo + separador;
            }
            graf.SetAttribute("MOTIVO", valMotivo);
            string valT1Motivo = "";
            foreach (string valorTlMotivo in lista_motivos)
            {
                if (!string.IsNullOrEmpty(valT1Motivo))
                    valT1Motivo += separador;
                
				valT1Motivo += valorTlMotivo;
            }
            graf.SetAttribute("TLMOTIVO", valT1Motivo);
            if (horarioConsulta)
                graf.SetAttribute("CHORARIO","S");
            if (picagemConsulta)
            {
                graf.SetAttribute("CPICAGEM", "S");
                graf.SetAttribute("PERIODO","S");
                graf.SetAttribute("CATRASO","S");
            }
            if (isDirigente.Equals("S"))
                graf.SetAttribute("CDIRIG","S");
            else
                graf.SetAttribute("CDIRIG","N");
            graf.SetAttribute("OBSERVAC",observacoes);

            rootNode.AppendChild(graf);
            return rootNode.OuterXml;
        }        
    }

	/// <summary>
	/// Classe que representa um pedido flash de picagem
	/// </summary>
	public abstract class RequestFlashAttendance
		: ExtControl
	{
        /// <summary>
        /// Enumerado com os tipos de comando possveis
        /// LoadFlash  o comando que corresponde ao pedido dos dados do flash
        /// GetGraficoXml  o comando que corresponde ao pedido to introduce as alteraes que o user forneceu
        /// </summary>
        public enum CommandType { LoadFlash, GetGraficoXml };

        /// <summary>
        /// Enumerado com os tipos de resposta possveis
        /// LoadGraficoXml  a resposta do servidor to carregar o grfico
        /// GetGraficoXml  a resposta ao xml que o servidor recebe com os dados alterados pelo user
        /// </summary>
        public enum CommandResponseType { LoadGraficoXml, GetGraficoXml };

        /// <summary>
        /// Type de comando do pedido
        /// </summary>
        private CommandType tipoComando;

        /// <summary>
        /// Parmetro do pedido
        /// </summary>
        private string parametro;

        /// <summary>
        /// Fields de historial do pedido
        /// </summary>
        private string[] camposHistorial;

        /// <summary>
        /// User em Sesso
        /// </summary>
        private User user;

        /// <summary>
        /// Funo que vai ser invocada to criar um flash picagem do tipo do pedido
        /// </summary>
        /// <param name="args">argumentos do pedido</param>
        /// <param name="utilizador">user</param>
        /// <returns>O Pedido flash vai ser inicializado</returns>
        public delegate RequestFlashAttendance CreatesFlashAttendance(string[] args,User user);
        
        /// <summary>
        /// Constructor da classe
        /// </summary>
        /// <param name="args">argumentos do pedido</param>
        /// <param name="utilizador">user em sesso</param>
        public RequestFlashAttendance(string[] args, User user)
        {
            int nrArgs = args.Length;//n de argumentos no pedido
            if (nrArgs < 3)//se tiver menos que 3  erro, 1- comando,  2 - parametro , 3 - name do grafico
				throw new BusinessException("Erro a carregar o Flash de picagem.", "RequestFlashAttendance.RequestFlashAttendance", "Insufficient number of arguments.");
            this.tipoComando = (CommandType)Enum.Parse(typeof(CommandType), args[0]);//tipo de comando
            this.parametro = args[1];//parmetro
            this.camposHistorial = new string[nrArgs - 3];//fields do historial
            for (int i = 3; i < nrArgs; i++)
            {
                this.camposHistorial[i - 3] = args[i];
                //this.camposHistorialResposta += "[" + args[i];
            }
            this.user = user;//user em sesso
        }

        /// <summary>
        /// Funo que executa o pedido
        /// </summary>
        /// <returns>a resposta ao pedido</returns>
        public override object processRequest()
        {
			List<string> response = new List<string>();
            try
            {
                switch (tipoComando)
                {
                    case CommandType.LoadFlash:
                        response.AddRange(new string[] { CommandResponseType.LoadGraficoXml.ToString(), sendFlash()});
                        break;						
                    case CommandType.GetGraficoXml:
                        response.AddRange(new string[] { CommandResponseType.GetGraficoXml.ToString(), receivesFlash()});
                        break;						
                    default:
						throw new BusinessException(null, "RequestFlashAttendance.processRequest", "Command not defined: " + tipoComando.ToString());
                }
				response.AddRange(this.camposHistorial);
                return response.ToArray();
            }
            catch (GenioException ex)
			{
				if (ex.ExceptionSite == "PedidoFlash${campo.Tgrafico}${campo.Form.ToUpper()}${campo.Fdbf.ToUpper()}${campo.Fcampo.ToUpper()}.processRequest")
					throw;
				throw new BusinessException(ex.UserMessage, "PedidoFlash${campo.Tgrafico}${campo.Form.ToUpper()}${campo.Fdbf.ToUpper()}${campo.Fcampo.ToUpper()}.processRequest", "Error processing Flash request: " + ex.Message, ex);
			}
            catch (Exception ex)
            {
                throw new BusinessException(null, "PedidoFlash${campo.Tgrafico}${campo.Form.ToUpper()}${campo.Fdbf.ToUpper()}${campo.Fcampo.ToUpper()}.processRequest", "Error processing Flash request: " + ex.Message, ex);
            }
        }    

        /// <summary>
        /// Mtodo que devolve ou coloca o name do comando
        /// </summary>
        public CommandType QCommandType
        {
            get { return tipoComando; }
        }

        /// <summary>
        /// Mtodo que devolve ou coloca o parametro do pedido
        /// </summary>
        public string Parameter
        {
            get { return parametro; }
        }

        /// <summary>
        /// Mtodo que devolve ou coloca os fields do historial
        /// </summary>
        public string[] HistoryFields
        {
            get { return camposHistorial; }
            set { camposHistorial = value; }
        }

        /// <summary>
        /// Mtodo que devolve ou coloca o user
        /// </summary>
        public User User
        {
            get { return user; }

        }
        
        /// <summary>
        /// Este mtodo corresponde ao pedido de estrutura do flash
        /// </summary>
        /// <returns>uma string com o xml que corresponde  estrutura do flash </returns>
        public abstract string sendFlash();

        /// <summary>
        /// Mtodo usado to responder aos pedidos LoadObject que correspondem aos registos
        /// que vo ser visualizados no flash
        /// </summary>
        /// <returns>uma string com o xml que corresponde aos registos que devem ser visualizados no flash</returns>
        public abstract string receivesFlash();
	}
}
