using CSGenio.persistence;
using NUnit.Framework;
using System;
using CSGenio.framework;
using System.Diagnostics;

namespace DbAdmin.IntegrationTest
{
    public class CreateDatabase
    {

        private IConfiguration config;


        [SetUp]
        public void Setup()
        {
            config = new ConfigurationBuilder()
                .AddJsonFile("appsettings.json")
                .Build();
            PersistenceFactoryExtension.Use();
        }

        [Test]
        public void AccessToServer()
        {
            PersistentSupport sp = PersistentSupport.getPersistentSupport(Configuration.DefaultYear);
            var connection = sp.GetConnectionToServer();
            connection.Open();
            Assert.IsTrue(connection.State == System.Data.ConnectionState.Open);
        }

        [Test]
        public void ConnectToDatabase()
        {
            PersistentSupport sp = PersistentSupport.getPersistentSupport(Configuration.DefaultYear);
            sp.openConnection();
            Assert.IsTrue(sp.Connection.State == System.Data.ConnectionState.Open);
        }

        private void AddDummySystem(string database) {
            var currentDataSystem = CSGenio.framework.Configuration.DataSystems[0];

            var dataSystem = currentDataSystem.ShallowCopy();
            dataSystem.Name = database;
            dataSystem.Schemas[0].Schema = database;
            CSGenio.framework.Configuration.DataSystems.Add(dataSystem);                
        }



        [Test]
        public void CreateDatabaseOnly()
        {
            string newDB = config["NewDB:DatabaseName"];
            AddDummySystem(newDB);
            PersistentSupport sp = PersistentSupport.getPersistentSupport(newDB);
            //Arrange: Delete existing databases if any
            if (sp.CheckIfDatabaseExists(newDB))
            {
                sp.Drop(newDB);
            }
            var datasystem = CSGenio.framework.Configuration.ResolveDataSystem(newDB, CSGenio.framework.Configuration.DbTypes.NORMAL);
            var user = datasystem.LoginDecode();
            var password = datasystem.PasswordDecode();
            var dbMaintenance = new DBMaintenance(AppDomain.CurrentDomain.BaseDirectory);

            //Act: Reindex
            var result = dbMaintenance.StartReindexation(user, password, "CREATEDB", null, "", false, year: newDB);
            
            var stopWatch = new Stopwatch();
            stopWatch.Start();
            
            int timeout = config.GetValue<int?>("NewDB:Timeout") ?? 60;
            while (!result.Progress.IsFinished() && stopWatch.Elapsed.TotalSeconds < timeout)
                Thread.Sleep(50);

            //Assert: Check if database exists
            Assert.IsTrue(sp.CheckIfDatabaseExists(newDB));
            Assert.AreEqual(ExecuteQueryCore.RdxProgressStatus.SUCCESS, result.Progress.State, result.Progress.Message);

            //Teardown
            sp.Drop(newDB);
        }


        [Test]
        public void CreateDatabaseDefaultScripts()
        {
            string newDb = config["NewDB:DatabaseName"];
            AddDummySystem(newDb);
            PersistentSupport sp = PersistentSupport.getPersistentSupport(newDb);
            //Arrange: Delete existing databases if any
            if (sp.CheckIfDatabaseExists(newDb))
            {
                sp.Drop(newDb);
            }
            var datasystem = CSGenio.framework.Configuration.ResolveDataSystem(newDb, CSGenio.framework.Configuration.DbTypes.NORMAL);
            var user = datasystem.LoginDecode();
            var password = datasystem.PasswordDecode();
            var dbMaintenance = new DBMaintenance(AppDomain.CurrentDomain.BaseDirectory);

            //Act: Reindex
            var result = dbMaintenance.StartReindexation(user, password, "", new System.Collections.Generic.List<string>(), "", false, year: newDb);
            
            var stopWatch = new Stopwatch();
            stopWatch.Start();
            var timeout = config.GetValue<int?>("NewDB:Timeout") ?? 60;
            while (!result.Progress.IsFinished() && stopWatch.Elapsed.TotalSeconds < timeout)
                Thread.Sleep(50);

            
            //Assert: Check if database exists
            Assert.IsTrue(sp.CheckIfDatabaseExists(newDb));
            Assert.AreEqual(ExecuteQueryCore.RdxProgressStatus.SUCCESS, result.Progress.State, result.Progress.Message);
            Assert.AreEqual(Configuration.VersionDbGen, Configuration.GetDbVersion(newDb));
            
            //Teardown
            sp.Drop(newDb);
        }
    }
}